from osgeo import gdal, gdal_array
import sys
import os
import numpy as np
from datetime import datetime

####################################################################################################################################################
# GLOFAS seasonal forecast processing script #5:
#
# For one forecast (basin averaged) calculate the maximum probability of exceedance across the 4-month forecast horizon
# Uses same criteria as fcprod3
#
#(file paths have been changed)
####################################################################################################################################################

#Function to take max probabilities for both 20th and 80th percentiles, and decide which is the one which will be displayed on the interface
def decide_max_prob(val80, val80week, val20, val20week):

    if val80 >= 50 and val20 >= 50:
        if val80week < val20week:
            return val80
        elif val20week < val80week:
            return -val20
        
    elif val20 >= 50 and val80 < 50:
        return -val20
    
    elif val80 >= 50 and val20 < 50:
        return val80
    
    elif val80 < 50 and val20 < 50:
        if val80 > val20:
            return val80
        elif val20 > val80:
            return -val20
        
#####################
# MAIN FUNCTION
#####################
           

def compute_ba_max_probs(year, month):
    
    bn = 306 #number of basins
        
    fdir = '/basin_averages/save_dir/'+str(year)+str(month).zfill(2)+'/'
    
    f80 = fdir+"ba_weekly_exc_probs"+str(year)+str(month).zfill(2)+"_pctl80.txt"
    data80 = np.genfromtxt(f80)
    probexc80 = data80[:,3:] #the 3: removes the 3 columns when copying it to the new array (this just holds the basin ID and first 2 weeks; which we aren't including for the maps)
    
    f20 = fdir+"ba_weekly_exc_probs"+str(year)+str(month).zfill(2)+"_pctl20.txt"
    data20 = np.genfromtxt(f20)
    probexc20 = data20[:,3:]
    
    Pmax_all=np.zeros(bn)
    Pmax80=np.zeros(bn)
    Pmax80=np.zeros(bn)
    
    for b in range(bn):

        maxprob80=max(probexc80[b,:])
        prob80w = np.where(probexc80[b,:] == max(probexc80[b,:]))

        maxprob20=max(probexc20[b,:])
        prob20w = np.where(probexc20[b,:] == max(probexc20[b,:]))
        
        Pmax_all[b] = decide_max_prob(maxprob80, prob80w[0], maxprob20, prob20w[0])
        
    
    #In the file, orange values will have negative probabilities, blue will have positive probabilities    
    savefile = fdir+'ba_max_probs_4mon_overview_'+str(year)+str(month).zfill(2)+'.txt'
    np.savetxt(savefile, Pmax_all, fmt="%s")
    
        
if __name__ == '__main__': 
    compute_ba_max_probs(int(sys.argv[1]), int(sys.argv[2]))

        
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
