from osgeo import gdal, gdal_array
import sys
import numpy as np
import os
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection
from matplotlib.patches import PathPatch
import matplotlib.patches as mpatches
from datetime import tzinfo, timedelta, datetime, date, time
import datetime as datetime
from dateutil.relativedelta import relativedelta
from matplotlib import rc

####################################################################################################
# GLOFAS seasonal forecast processing script #8:
#
# Plot a hydrograph (provide script with lat and lon) for one reporting point
#
#(file paths have been changed)
#####################################################################################################


#Function to open a raster file and put the data into an array
def open_raster_file(fdir, fname):
    filename = fdir+fname
    ff = gdal.Open(filename)
    rows = ff.RasterYSize
    cols = ff.RasterXSize
    band = ff.GetRasterBand(1)
    data = band.ReadAsArray(0,0,cols,rows)
    ff = None               
    
    return data

def plot_hydrograph(year, month, xloc, yloc): #lon, later
    
    en = 51

    fcstdir = "/weekly_averages/save_dir/"
    thresdir = "/archive/ref_clim/percentiles/"+str(month).zfill(2)+"/"
    
    savedir = "/map_vis/save_dir/"+str(year)+str(month).zfill(2)+"/"
    if not os.path.exists(savedir):
        os.makedirs(savedir)
    

    
    # convert the given lat and lon values to the index we should use
    lats = np.arange(90,-60,-0.1)
    lons = np.arange(-180,180,0.1)    
    y = (np.abs(lats[:]-yloc)).argmin()
    x = (np.abs(lons[:]-xloc)).argmin() 
    
    datei = datetime.date(year,month,1)
    months = ['Jan.','Feb.','Mar.','Apr.','May.','Jun.','Jul.','Aug.','Sep.','Cct.','Nov.','Dec.','Jan.','Feb.','Mar.']

    
    # to hold all ensemble members and all weeks for the point we're looking at
    fcst_array=np.zeros((en,17))  
    
    #to hold the threshold values for each week out to 17 weeks, for the point we're looking at:
    normcond_array=np.zeros(17)
    p90_array=np.zeros(17)
    p10_array=np.zeros(17)
    
    # to hold the values for each line of the fan plot in the final hydrograph:
    fp100=np.zeros(17)
    fp90=np.zeros(17)
    fp75=np.zeros(17)
    fp50=np.zeros(17)
    fp25=np.zeros(17)
    fp10=np.zeros(17)
    fp0=np.zeros(17)

    #load in the climatological "normal conditions" and percentiles for this time of year, and save the ones for the point we're looking at:
    print 'opening files...'
    for w in range(17):
        
        
        normcond=open_raster_file(thresdir,"clim_m"+str(month).zfill(2)+"_w"+str(w+1).zfill(2)+"_p50.map")
        normcond_array[w] = normcond[y,x]
        
        p90=open_raster_file(thresdir,"clim_m"+str(month).zfill(2)+"_w"+str(w+1).zfill(2)+"_p90.map")
        p90_array[w] = p90[y,x]
        
        p10=open_raster_file(thresdir,"clim_m"+str(month).zfill(2)+"_w"+str(w+1).zfill(2)+"_p10.map")                            
        p10_array[w] = p10[y,x]

        #load in the forecast data file for each ensemble member and put the data for all ensemble members for this basin into the fcst_array
        for e in range(en):

            fcstdatagrid=open_raster_file(fcstdir,"waver_"+str(year)+str(month).zfill(2)+"01_p"+str(e).zfill(2)+"_w"+str(w+1).zfill(2)+".map")
            fcst_array[e,w] = fcstdatagrid[y,x] 

        fp100[w] = np.percentile(fcst_array[:,w],100)
        fp90[w] = np.percentile(fcst_array[:,w],90)
        fp75[w] = np.percentile(fcst_array[:,w],75)
        fp50[w] = np.percentile(fcst_array[:,w],50)
        fp25[w] = np.percentile(fcst_array[:,w],25)
        fp10[w] = np.percentile(fcst_array[:,w],10)
        fp0[w] = np.percentile(fcst_array[:,w],0)

    
    fig, ax = plt.subplots()
    fig.set_size_inches(18, 10)

    #The function "smooth" does a 3-week moving average of the data - this is used to plot the smoothed versions of the percentile thresholds
    def smooth(y):
        box_pts = 3 
        box = np.ones(box_pts)/box_pts
        y_smooth = np.convolve(y, box, mode='same')
        return y_smooth
    
    x=range(17)

    ax.set_xlim([0,16])
    
    print 'calculating moving averages'
    #Fudging the moving average for the last week because we don't have an extra week after the end of the forecast to use to get the last week
    p20_array[:] = smooth(p20_array[:])
    p20_array[16] = (p20_array[15]+(p20_array[15]-p20_array[14]))
    
    p80_array[:] = smooth(p80_array[:])
    p80_array[16] = (p80_array[15]+(p80_array[15]-p80_array[14]))
    
    normcond_array[:] = smooth(normcond_array[:])
    normcond_array[16] = (normcond_array[15]+(normcond_array[15]-normcond_array[14]))
    
    print 'plotting stuff'
  
    ax.fill_between(range(17), p10_array[:], p90_array[:], color='k',alpha=0.01,edgecolor='None',lw=0) 

    ax.plot(range(17), normcond_array[:],'k--',lw=0.75,zorder=1) 

    ax.fill_between(x, fp90[:], fp100[:], color='k',alpha=0.3,edgecolor='None',zorder=8)
    ax.fill_between(x, fp75[:], fp90[:], color='k',alpha=0.5,edgecolor='None',zorder=8)
    ax.fill_between(x, fp50[:], fp75[:], color='k',alpha=0.6,edgecolor='None',zorder=8)
    ax.fill_between(x, fp25[:], fp50[:], color='k',alpha=0.6,edgecolor='None',zorder=8)
    ax.fill_between(x, fp10[:], fp25[:], color='k',alpha=0.5,edgecolor='None',zorder=8)
    ax.fill_between(x, fp0[:], fp10[:], color='k',alpha=0.3,edgecolor='None',zorder=8)
    l50 = ax.plot(x, fp50[:], 'k',zorder=9)
    
    auto_ylimmin,auto_ylimmax = ax.get_ylim()
    

    ax.fill_between(range(17), p90_array[:], auto_ylimmax, color='b', hatch="\\", edgecolor="b", linewidth=0.0,alpha=0.1,lw=0,zorder=1) 
    ax.fill_between(range(17), auto_ylimmin, p10_array[:], color='#e87c08', hatch="\\", edgecolor="#e87c08", linewidth=0.0,alpha=0.15,lw=0,zorder=1)
       

    #Shade over the first 10 days in semi-transparent white, because the forecast won't come out until 10th but want to see the overall shape so don't want to remove entirely
    plt.axvspan(0,1.4285,facecolor='w',edgecolor='None',alpha=0.7,zorder=10)

    plt.ylabel(r'River Flow$^{*}$ (m$^{3}$/s)', fontsize = 14)
    
    start_date = str(year)+'/'+str(month).zfill(2)+'/01'
    datelist = [0] * 17
    date_1 = datetime.datetime.strptime(start_date, "%Y/%m/%d").date()
    print date_1
    datelist[0] = str(date_1.strftime('%d'))
    week_id = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]
    datedeltas = [7,14,21,28,35,42,49,56,63,70,77,84,91,98,105,112]
    for i, d in zip(week_id, datedeltas):
        next_date = date_1 + datetime.timedelta(days=d)
        datelist[i] = str(next_date.strftime('%d'))
       
    month2date = date_1 + relativedelta(months=1)
    month2 = (month2date - date_1).days
    month2x = month2/7.0
    month2loc = month2x/16.0
    print month2date, month2, month2x, month2loc
    
    month3date = date_1 + relativedelta(months=2)
    month3 = (month3date - date_1).days
    month3x = month3/7.0
    month3loc = month3x/16.0
    print month3date, month3, month3x, month3loc
    
    month4date = date_1 + relativedelta(months = 3)
    month4 = (month4date - date_1).days
    month4x = month4/7.0
    month4loc = month4x/16.0
    print month4date, month4, month4x, month4loc
    
    plt.annotate('', xy=(month2loc, 0), xycoords='axes fraction', xytext=(month2loc,-0.08), textcoords='axes fraction', arrowprops=dict(arrowstyle="-"))
    plt.annotate('', xy=(month3loc, 0), xycoords='axes fraction', xytext=(month3loc,-0.08), textcoords='axes fraction', arrowprops=dict(arrowstyle="-"))
    plt.annotate('', xy=(month4loc, 0), xycoords='axes fraction', xytext=(month4loc,-0.08), textcoords='axes fraction', arrowprops=dict(arrowstyle="-"))
        
    #plot week start dates on x axis
    plt.xticks(x, datelist, fontsize = 12)

    #add note about start dates
    plt.annotate(r'$^{*}$River flow is a weekly average, displayed at start of week (dates shown)',xy=(.265,-0.12),xytext=(.265,-0.12),xycoords='axes fraction',ha='center',va='bottom', fontsize =12)
    
    #add note about start dates
    plt.annotate('Please note:\nThis is an experimental\nforecast product',xy=(1.13,0.86),xytext=(1.13,0.86),xycoords='axes fraction',ha='center',va='bottom', fontsize =14, bbox=dict(boxstyle="round,pad=0.5", edgecolor='red', lw=2, facecolor='white'))

    #add the months to the x axis; with a catch incase the month initialised is 12
    plt.annotate(str(months[datei.month-1])+" "+str(datei.year),xy=(.125,-.075),xytext=(.125,-.075),xycoords='axes fraction',ha='center',va='bottom')#,arrowprops=dict(arrowstyle='-[,widthB=8.2,lengthB=.5',lw=1)

    #catch to change the year on the labels if the forecast includes Dec and Jan
    if datei.month == 12:
        plt.annotate(str(months[datei.month])+" "+str(datei.year+1),xy=(.375,-.075),xytext=(.375,-.075),xycoords='axes fraction',ha='center',va='bottom') #,arrowprops=dict(arrowstyle='-[,widthB=8.2,lengthB=.5',lw=1)
        plt.annotate(str(months[datei.month+1])+" "+str(datei.year+1),xy=(.625,-.075),xytext=(.625,-.075),xycoords='axes fraction',ha='center',va='bottom') #,arrowprops=dict(arrowstyle='-[,widthB=8.2,lengthB=.5',lw=1)
        plt.annotate(str(months[datei.month+2])+" "+str(datei.year+1),xy=(.875,-.075),xytext=(.875,-.075),xycoords='axes fraction',ha='center',va='bottom') #,arrowprops=dict(arrowstyle='-[,widthB=8.2,lengthB=.5',lw=1)
    else:
        plt.annotate(str(months[datei.month])+" "+str(datei.year),xy=(.375,-.075),xytext=(.375,-.075),xycoords='axes fraction',ha='center',va='bottom') #,arrowprops=dict(arrowstyle='-[,widthB=8.2,lengthB=.5',lw=1)
        plt.annotate(str(months[datei.month+1])+" "+str(datei.year),xy=(.625,-.057),xytext=(.625,-.075),xycoords='axes fraction',ha='center',va='bottom') #,arrowprops=dict(arrowstyle='-[,widthB=8.2,lengthB=.5',lw=1)
        plt.annotate(str(months[datei.month+2])+" "+str(datei.year),xy=(.875,-.075),xytext=(.875,-.075),xycoords='axes fraction',ha='center',va='bottom') #,arrowprops=dict(arrowstyle='-[,widthB=8.2,lengthB=.5',lw=1)

    # Add a legend 
    #Title1 = plt.Line2D((0,0),(0,0), color='w', label='Historical Data:')
    #Pctl90 = mpatches.Patch(color='blue', alpha=0.18, hatch='\\',label='  > 90th Percentile')
    #Pctl10 = mpatches.Patch(color='#e87c08',alpha=0.22, hatch='\\',label='  < 10th Percentile')
    #LineMax = plt.Line2D((0,1),(0,0), color='blue',ls='-.',label='  Max in model record')
    #LineMin = plt.Line2D((0,1),(0,0), color='#e87c08',ls='-.',label='  Min in model record')
    #NormLine = plt.Line2D((0,1),(0,0), color='k', linestyle='--', label='  Average Conditions')
    #Gap = plt.Line2D((0,1),(0,0), color='w', label=' ')

    #Title2 = plt.Line2D((0,1),(0,0), color='w', label='Forecast:')
    #MedLine = plt.Line2D((0,1),(0,0), color='k', lw=1.5, label='  Median')
    #Fcst2575 = mpatches.Patch(color='k', alpha=0.5, label='  25-75th Percentile')
    #Fcst7590 = mpatches.Patch(color='k', alpha=0.3, label='  10-90th Percentile')
    #Fcst100 = mpatches.Patch(color='k', alpha=0.2, label='  0-100th Percentile')

    box = ax.get_position()
    ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
    
    #leg1=ax.legend(handles=[Title2,MedLine,Fcst2575,Fcst7590,Fcst100,Gap,Title1,Pctl90,Pctl10,NormLine], fontsize = 14, loc='center right', bbox_to_anchor=(1.26, 0.6), frameon=False)

    plt.savefig(savedir+'/hydrograph_'+str(year)+str(month).zfill(2)+'_loc_'+str(xloc)+'x_'+str(yloc)+'y.png', bbox_inches='tight',dpi=200)


if __name__ == '__main__': 
    plot_hydrograph(int(sys.argv[1]), int(sys.argv[2]), float(sys.argv[3]), float(sys.argv[4]))
